package com.organizedCampgrounds.proxy;

import java.net.*; 
import org.apache.soap.*;  
import org.apache.soap.rpc.*; 
import org.apache.soap.encoding.*;
import org.apache.soap.encoding.soapenc.*;
import org.apache.soap.util.xml.QName;

import com.organizedCampgrounds.*;

    public class CampgroundProxy implements Campground { 
      private URL url;
      private String urn = "urn:Campground"; ;
      private Call call;

      public CampgroundProxy() throws Exception { 
        /*
         * Define the location where the campground may be found and
         * the identifier for the campground object.
         */
        call = new Call();
        call.setTargetObjectURI( urn ); 
        call.setEncodingStyleURI( Constants.NS_URI_SOAP_ENC ); 
      } 

      public void setEndPoint(URL _url){
        this.url = _url;
      }

      public int computePrice(int dayOfWeek){
        int price=0;
        java.util.Vector params = new java.util.Vector(); 

        try { 
          System.out.println("Contacting campground");
          System.out.println(" URL: " + url);
          System.out.println(" URN: " + urn); 

          /*
           * Call computePrice
           */
          call.setMethodName( "computePrice" ); 
          params.addElement( new Parameter( "dayOfWeek", Integer.class, new Integer(dayOfWeek), null ) ); 
          call.setParams( params ); 
          Response response = call.invoke( url, "" ); 
          if( !response.generatedFault()){ 
            Parameter result = response.getReturnValue(); // response was OK 
            price = ((Integer)(result.getValue())).intValue();
          } 
          else { 
            Fault f = response.getFault(); 
            System.err.println("Campground error: \n " + 
                f.getFaultCode() + "\n " + f.getFaultString()); 
          } 
        }
        catch( SOAPException e ) { 
            /*
             * Error trying to make the call
             */
		Throwable target = e.getTargetException();
		System.out.println("Call error: " + target.getMessage());
	  	target.printStackTrace();
            System.err.println("SOAPException: ");
            System.err.println(" " + e.getFaultCode());  
            System.err.println(" " + e.getMessage()); 
        } 
        return price;
      }

      public int getNumberOfCampsites(){
        int nCampsites=0;
        java.util.Vector params = new java.util.Vector(); 
        try{
          /*
           * Call getNumberOfCampsites
           */
          call.setMethodName( "getNumberOfCampsites" ); 
          params.clear();
          call.setParams( params ); 
          Response response = call.invoke( url, "" ); 
          if( !response.generatedFault()){ 
            Parameter result = response.getReturnValue(); // response was OK 
            nCampsites = ((Integer)(result.getValue())).intValue();
          } 
          else { 
            Fault f = response.getFault(); 
            System.err.println("Campground error: \n " + 
                f.getFaultCode() + "\n " + f.getFaultString()); 
          } 
        }
        catch( SOAPException e ) { 
            /*
             * Error trying to make the call
             */
		Throwable target = e.getTargetException();
		System.out.println("Call error: " + target.getMessage());
	  	target.printStackTrace();
            System.err.println("SOAPException: ");
            System.err.println(" " + e.getFaultCode());  
            System.err.println(" " + e.getMessage()); 
        } 
        return nCampsites;
      }




      public Campsite getCampsite(int campsiteNumber){
        Campsite campsite=null;
        java.util.Vector params = new java.util.Vector(); 
        try{
          /*
           * Call getCampsite for specified campsite
           */
          call.setMethodName( "getCampsite" ); 
          SOAPMappingRegistry registry = new SOAPMappingRegistry();
          QName qname = new QName( "http://proxy.organizedCampgrounds.com", "Campsite" );
          BeanSerializer serializer = new BeanSerializer();
          registry.mapTypes( Constants.NS_URI_SOAP_ENC, qname, Campsite.class, serializer, serializer );
          call.setSOAPMappingRegistry( registry );

          params.clear();
          params.addElement( new Parameter( "campsiteNumber", Integer.class, new Integer(campsiteNumber), null ) ); 
          call.setParams( params ); 
          Response response = call.invoke( url, "" ); 
          if( !response.generatedFault()){ 
            Parameter result = response.getReturnValue(); // response was OK 
            campsite = ((Campsite)(result.getValue()));
          } 
          else { 
            Fault f = response.getFault(); 
            System.err.println("Campground error: \n " + 
              f.getFaultCode() + "\n " + f.getFaultString()); 
          } 
        } 
        catch( SOAPException e ) { 
            /*
             * Error trying to make the call
             */
		Throwable target = e.getTargetException();
		System.out.println("Call error: " + target.getMessage());
	  	target.printStackTrace();
            System.err.println("SOAPException: ");
            System.err.println(" " + e.getFaultCode());  
            System.err.println(" " + e.getMessage()); 
        } 
        return campsite;
      } 
    

      public boolean reserveCampsite(int dayOfWeek, int campsiteNumber, String name){
        boolean success=false;
        java.util.Vector params = new java.util.Vector(); 

        try { 
          /*
           * Call reserveCampsite
           */
          call.setMethodName( "reserveCampsite" ); 
          params.addElement( new Parameter( "dayOfWeek", Integer.class, new Integer(dayOfWeek), null ) ); 
          params.addElement( new Parameter( "campsiteNumber", Integer.class, new Integer(campsiteNumber), null ) ); 
          params.addElement( new Parameter( "name", String.class, name, null ) ); 
          call.setParams( params ); 
          Response response = call.invoke( url, "" ); 
          if( !response.generatedFault()){ 
            Parameter result = response.getReturnValue(); // response was OK 
            success = ((Boolean)(result.getValue())).booleanValue();
          } 
          else { 
            Fault f = response.getFault(); 
            System.err.println("Campground error: \n " + 
                f.getFaultCode() + "\n " + f.getFaultString()); 
          } 
        }
        catch( SOAPException e ) { 
            /*
             * Error trying to make the call
             */
		Throwable target = e.getTargetException();
		System.out.println("Call error: " + target.getMessage());
	  	target.printStackTrace();
            System.err.println("SOAPException: ");
            System.err.println(" " + e.getFaultCode());  
            System.err.println(" " + e.getMessage()); 
        } 
        return success;
      }
    }

