package com.organizedCampgrounds.proxy;

import java.io.*;
import java.util.zip.*;

import org.w3c.dom.*;
import org.apache.soap.*;
import org.apache.soap.encoding.soapenc.*;
import org.apache.soap.rpc.*;
import org.apache.soap.util.*;
import org.apache.soap.util.xml.*;

/**
   Serializer for Campsite class which compresses the description string.
 */
public class CampsiteCompSerializer implements Serializer, Deserializer{

  public void marshall(String inScopeEncStyle, Class javaType,
    Object src, Object context, Writer sink,
    NSStack nsStack, XMLJavaMappingRegistry xjmr,
    SOAPContext ctx)
        throws IllegalArgumentException, IOException{

    nsStack.pushScope();

    /*
     * Start the Campsite element.
     */
    SoapEncUtils.generateStructureHeader(inScopeEncStyle, javaType, context,
      sink, nsStack, xjmr);

    sink.write(StringUtils.lineSeparator);

    com.organizedCampgrounds.Campsite src2 =
      (com.organizedCampgrounds.Campsite)src;
    Parameter param;

    /*
     * The description string is compressed before inserting it into the
     * Parameter.
     */
    param = new Parameter("description", java.lang.String.class,
      compressIt(src2.getDescription()), null);
    xjmr.marshall(inScopeEncStyle, Parameter.class, param, null,
      sink, nsStack, ctx);
    sink.write(StringUtils.lineSeparator);

    /*
     * Size is an integer.
     */
    param = new Parameter("size", int.class, new Integer(src2.getSize()), null);
    xjmr.marshall(inScopeEncStyle, Parameter.class, param, null,
      sink, nsStack, ctx);
    sink.write(StringUtils.lineSeparator);

    /*
     * Shade is an integer.
     */
    param = new Parameter("shade", int.class, new Integer(src2.getShade()),
      null);
    xjmr.marshall(inScopeEncStyle, Parameter.class, param, null,
      sink, nsStack, ctx);
    sink.write(StringUtils.lineSeparator);

    sink.write("</" + context + '>');

    nsStack.popScope();
  }

  public Bean unmarshall(String inScopeEncStyle, QName elementType,
    Node src, XMLJavaMappingRegistry xjmr,
    SOAPContext ctx)
    throws IllegalArgumentException{

    Element root = (Element)src;
    Element tempEl = DOMUtils.getFirstChildElement(root);
    com.organizedCampgrounds.proxy.Campsite target;

    try{
      /*
       * Create a Campsite object.
       */
      target =
        (com.organizedCampgrounds.proxy.Campsite)com.organizedCampgrounds.Campsite.class.newInstance();
    }
    catch (Exception e){
      throw new IllegalArgumentException("Error constructing object: "
        + e.getMessage());
    }

    while (tempEl != null){
      Bean paramBean = xjmr.unmarshall(inScopeEncStyle,
        RPCConstants.Q_ELEM_PARAMETER,
        tempEl,
        ctx);
      Parameter param = (Parameter)paramBean.value;
      String tagName = tempEl.getTagName();

      /*
       * Decompress the description string.
       */
      if (tagName.equals("description")){
        target.description_Elem = decompressIt((java.lang.String)param.getValue());
      }

      /*
       * Size is an integer.
       */
      if (tagName.equals("size")){
        target.size_Elem = ((Integer)param.getValue()).intValue();
      }

      /*
       * Shade is an integer.
       */
      if (tagName.equals("shade")){
        target.shade_Elem = ((Integer)param.getValue()).intValue();
      }

      /*
       * Get the next parameter.
       */
      tempEl = DOMUtils.getNextSiblingElement(tempEl);
    }

    return new Bean(com.organizedCampgrounds.Campsite.class, target);
  }


  /*
   * Compress a string.
   */
  private String compressIt(String in){
        byte[] uncompressedBytes = in.getBytes();
            System.out.println("Original length: " + uncompressedBytes.length);
            Deflater def = new Deflater();
            def.setInput(uncompressedBytes);
            def.finish();
            byte outbuf[] = new byte[uncompressedBytes.length];
            int len = def.deflate(outbuf);
            System.out.println("Compressed length: " + len);
            byte[] compressedBytes = new byte[len];
            for(int i=0; i<len; i++)
                compressedBytes[i] = outbuf[i];
            return new String(compressedBytes);
  }
  
  /*
   * Decompress a string.
   */  
  private String decompressIt(String compressedBytes){
            Inflater inf = new Inflater();
            inf.setInput(compressedBytes.getBytes());
            byte infbuf[] = new byte[inf.getRemaining()*3];
            try{
                System.out.println("Inflated length: " + inf.inflate(infbuf));
            }
            catch(DataFormatException e){
                System.err.println("Err: " + e.getMessage());
            }
            return new String(infbuf);            
        }
  
}
