import com.hp.uddi.api.*;
import java.net.*;

/** FindBusiness
 * This class does a find business by business name. Built using HP
 * UDDI client SDK
 */
public class FindBusiness {
  /* URL for UDDI Inquiry in the Production UDDI registry node at IBM       */
  public final static String IBM_INQUIRY_URL_PRODUCTION = "http://www-3.ibm.com/services/uddi/inquiryapi";

  /* URL for UDDI Inquiry in the Test UDDI registry node at IBM             */
  public final static String IBM_INQUIRY_URL_TEST = "http://www-3.ibm.com/services/uddi/testregistry/inquiryapi";

  /* URL for UDDI Inquiry in the Production UDDI registry node at Microsoft */
  public final static String MICROSOFT_INQUIRY_URL_PRODUCTION = "http://uddi.microsoft.com/inquire";

  /* URL for UDDI Inquiry in the Test UDDI registry node at Microsoft       */
  public final static String MICROSOFT_INQUIRY_URL_TEST = "http://test.uddi.microsoft.com/inquire";

  // Print error messsage and die
  private static void fatal (String message) {
    System.err.println ("Fatal: " + message);
    System.exit (1);
  }

  public static void main (String args[]) {
     if (args.length != 2) 
       fatal (
       "usage: java com.hp.uddi.samples.FindBusiness [IBM_PROD|MS_PROD|IBM_TEST|MS_TEST|<url>] <business name>");
     UDDISOAPClient uddiSoapClient = null;

     String inquiryUrl = null;

     if (args[0].equals ("IBM_PROD"))
       inquiryUrl = IBM_INQUIRY_URL_PRODUCTION;
     else if (args[0].equals ("MS_PROD"))
       inquiryUrl = MICROSOFT_INQUIRY_URL_PRODUCTION;
     else if (args[0].equals ("IBM_TEST"))
       inquiryUrl = IBM_INQUIRY_URL_TEST;
     else if (args[0].equals ("MS_TEST"))
       inquiryUrl = MICROSOFT_INQUIRY_URL_TEST;
     else
       inquiryUrl = args[0];

     try {
       uddiSoapClient = new UDDISOAPClient (inquiryUrl);
       System.out.println ("Setting URL to " + inquiryUrl);
     } catch (MalformedURLException e) {
       fatal ("Malformed Inquiry URL " + inquiryUrl + ": "
              + e.getMessage());
     }
   
     FindBusinessRequest findBusinessRequest = new FindBusinessRequest();
     try {
       findBusinessRequest.setName (args[1]);
     } catch (UDDIException e) {
       fatal ("Error setting name : " + e.getErrorCode()
              + ": " + e.getErrorMessage());
     }

     UDDIResponse uddiResponse = null;
     try {
       System.out.println ("Request is " + findBusinessRequest.toXML());
       uddiResponse = uddiSoapClient.findBusiness (findBusinessRequest);
     } catch (UDDIException e) {
       fatal ("Error finding business: " + e.getErrorCode()
              + ": " + e.getErrorMessage());
     }

     System.out.println ("Got message: \n"
                         + uddiResponse.toXML ());
  }
}
