package wrox.webservices.UDDIImpl;

import java.net.*;
import java.util.*;
import java.io.*;
import javax.xml.parsers.*;

import com.ibm.uddi.*;
import com.ibm.uddi.client.*;
import com.ibm.uddi.response.*;
import com.ibm.uddi.datatype.tmodel.*;
import com.ibm.uddi.datatype.business.*; 
import com.ibm.uddi.util.*;

import org.apache.soap.*;
import org.apache.soap.transport.http.SOAPHTTPConnection;
import org.w3c.dom.*;
import org.apache.xml.serialize.*;
import org.apache.xerces.dom.*;
import org.xml.sax.InputSource;
import org.apache.xerces.parsers.DOMParser;

/**
 * The UddiBrowser class is a text based UDDI registry browser
 */
public class UddiBrowser {
  private static UDDIProxy uddiProxy;

  /* Browse by Business Name- return all matching business entities */
  private static String browseBusinessByName (String businessName) {
    BusinessList businessList = null;
    try {
        businessList = uddiProxy.find_business (businessName, null, 0);
    } catch (UDDIException e) {
      DispositionReport dispositionReport = e.getDispositionReport ();
      System.err.println ("Error finding businesses: " 
                          + dispositionReport.getErrCode() + ": "
			  + dispositionReport.getErrInfoText());
      System.exit (1); /* fatal error */
    } catch (SOAPException e) {
      System.err.println ("SOAP related error finding businesses: " 
                          + e.getFaultCode() + ": " + e.getMessage());
      System.exit (1); /* fatal error */

    }

    /* A bit of a hack to create a dummy element, and
     * get the businessList saved as a DOM element.
     */
    DocumentImpl doc         = new DocumentImpl (null);
    ElementImpl  elem        = new ElementImpl (doc, "dummyRootElement");
    businessList.saveToXML (elem);
    Node         uddiDocNode = elem.getFirstChild ();

    String xmlString = null;
    try {
      xmlString = Xml2String (uddiDocNode);
    } catch (Exception e) {
      e.printStackTrace ();
      fatal ("Internal Error: Converting businessList to XML document " 
             + e.getMessage ());
    }
    return xmlString;
  }

  /* Browse by Business key- return an entire businessEntity */
  private static String browseBusinessByKey (String businessKey) {
    Vector vector = new Vector ();
    vector.addElement (businessKey);

    BusinessDetail businessDetail = null;
    try {
        businessDetail = uddiProxy.get_businessDetail (vector);
    } catch (UDDIException e) {
      DispositionReport dispositionReport = e.getDispositionReport ();
      System.err.println ("Error getting business detail: " 
                          + dispositionReport.getErrCode() + ": "
			  + dispositionReport.getErrInfoText());
      System.exit (1); /* fatal error */
    } catch (SOAPException e) {
      System.err.println ("SOAP related error getting business detail: " 
                          + e.getFaultCode() + ": " + e.getMessage());
      System.exit (1); /* fatal error */

    }

    /* A bit of a hack to create a dummy element, and
     * get the businessDetail saved as a DOM element.
     */
    DocumentImpl doc         = new DocumentImpl (null);
    ElementImpl  elem        = new ElementImpl (doc, "dummyRootElement");
    businessDetail.saveToXML (elem);
    Node         uddiDocNode = elem.getFirstChild ();

    String xmlString = null;
    try {
      xmlString = Xml2String (uddiDocNode);
    } catch (Exception e) {
      e.printStackTrace ();
      fatal ("Internal Error: Converting businessDetail to XML document " 
             + e.getMessage ());
    }
    return xmlString;
  }

  /* Browse by Service key- return an entire businessService */
  private static String browseServiceByKey (String businessServiceKey) {
    Vector vector = new Vector ();
    vector.addElement (businessServiceKey);

    ServiceDetail serviceDetail = null;
    try {
        serviceDetail = uddiProxy.get_serviceDetail (vector);
    } catch (UDDIException e) {
      DispositionReport dispositionReport = e.getDispositionReport ();
      System.err.println ("Error getting service detail: " 
                          + dispositionReport.getErrCode() + ": "
			  + dispositionReport.getErrInfoText());
      System.exit (1); /* fatal error */
    } catch (SOAPException e) {
      System.err.println ("SOAP related error getting service detail: " 
                          + e.getFaultCode() + ": " + e.getMessage());
      System.exit (1); /* fatal error */
    }

    /* A bit of a hack to create a dummy element, and
     * get the businessDetail saved as a DOM element.
     */
    DocumentImpl doc         = new DocumentImpl (null);
    ElementImpl  elem        = new ElementImpl (doc, "dummyRootElement");
    serviceDetail.saveToXML (elem);
    Node         uddiDocNode = elem.getFirstChild ();

    String xmlString = null;
    try {
      xmlString = Xml2String (uddiDocNode);
    } catch (Exception e) {
      e.printStackTrace ();
      fatal ("Internal Error: Converting businessDetail to XML document " 
             + e.getMessage ());
    }
    return xmlString;
  }


  /* Browse by tModel Name- return all matching tModel infos */
  private static String browseTModelByName (String tModelName) {
    TModelList tModelList = null;
    try {
        tModelList = uddiProxy.find_tModel (tModelName, null, 0);
    } catch (UDDIException e) {
      DispositionReport dispositionReport = e.getDispositionReport ();
      System.err.println ("Error finding tModels: " 
                          + dispositionReport.getErrCode() + ": "
			  + dispositionReport.getErrInfoText());
      System.exit (1); /* fatal error */
    } catch (SOAPException e) {
      System.err.println ("SOAP related error finding tModels: " 
                          + e.getFaultCode() + ": " + e.getMessage());
      System.exit (1); /* fatal error */

    }

    /* A bit of a hack to create a dummy element, and
     * get the tModelList saved as a DOM element.
     */
    DocumentImpl doc         = new DocumentImpl (null);
    ElementImpl  elem        = new ElementImpl (doc, "dummyRootElement");
    tModelList.saveToXML (elem);
    Node         uddiDocNode = elem.getFirstChild ();

    String xmlString = null;
    try {
      xmlString = Xml2String (uddiDocNode);
    } catch (Exception e) {
      e.printStackTrace ();
      fatal ("Internal Error: Converting tModelList to XML document " 
             + e.getMessage ());
    }
    return xmlString;
  }


  /* Browse by tModel key- return a TModelDetail structure */
  private static String browseTModelByKey (String tModelKey) {
    Vector vector = new Vector ();
    vector.addElement (tModelKey);

    TModelDetail tModelDetail = null;
    try {
        tModelDetail = uddiProxy.get_tModelDetail (vector);
    } catch (UDDIException e) {
      DispositionReport dispositionReport = e.getDispositionReport ();
      System.err.println ("Error getting tModel detail: " 
                          + dispositionReport.getErrCode() + ": "
			  + dispositionReport.getErrInfoText());
      System.exit (1); /* fatal error */
    } catch (SOAPException e) {
      System.err.println ("SOAP related error getting tModel detail: " 
                          + e.getFaultCode() + ": " + e.getMessage());
      System.exit (1); /* fatal error */
    }


    TModel tModel = (TModel) tModelDetail.getTModelVector().elementAt(0);

    /* A bit of a hack to create a dummy element, and
     * get the tModelDetail saved as a DOM element.
     */
    DocumentImpl doc         = new DocumentImpl (null);
    ElementImpl  elem        = new ElementImpl (doc, "dummyRootElement");
    tModelDetail.saveToXML (elem);
    Node         uddiDocNode = elem.getFirstChild ();


    String xmlString = null;
    try {
      xmlString = Xml2String (uddiDocNode);
    } catch (Exception e) {
      e.printStackTrace ();
      fatal ("Internal Error: Converting tModelDetail to XML document " 
             + e.getMessage ());
    }
    return xmlString;
  }


  /* Print usage message */
  private static void usage () {
    System.out.println (
       "usage: UddiBrowser <inquiry_url> <find_business|find_service|find_tmodel> <options>\n"
     + "       where <inquiry_url> is one of IBM_TEST|IBM_PROD|MS_TEST|MS_PROD|url\n"
     + "             <options> are :\n"
     + "             [for find_business]\n"
     + "             name <partial_business_name>\n"
     + "             key  <businessKey>\n\n"
     + "             [for find_service]\n"
     + "             key  <serviceKey>\n\n"
     + "             [for find_tmodel]\n"
     + "             name  <partial_tmodel_name>\n"
     + "             key  <tModelKey>"
                        );
  }

  /* Print fatal error message and die */
  private static void fatal (String message) {
    System.err.println ("Fatal: " + message);
    System.exit (1);
  }

  /* Print informational message */
  private static void info (String message) {
    System.out.println ("Info: " + message);
  }

  /* Helper method for converting a DOM node to its 
   * string representation */
  private static String Xml2String (Node node) throws Exception {
    /* Setup the output formatter */
    OutputFormat outputFormat = new OutputFormat();
    outputFormat.setMethod        (Method.XML);
    outputFormat.setEncoding      ("ISO-8859-1");
    outputFormat.setIndenting     (true);
    outputFormat.setIndent        (2);
    outputFormat.setLineWidth     (0);
    outputFormat.setPreserveSpace (false);

    /* Setup the XML serializer */
    StringWriter  xmlWriter     = new StringWriter  ();
    XMLSerializer xmlSerializer = new XMLSerializer (xmlWriter, outputFormat);
    xmlSerializer.serialize ((Element)node);

    return xmlWriter.toString();
  }


  public static void main (String args[]) {

    /* Process command line arguments */
    if (args.length < 2) {
      usage ();
      System.exit (1);
    }

    String inquiryUrl;

    if (args[0].equals ("IBM_TEST"))
      inquiryUrl = UDDIOperatorURLs.IBM_INQUIRY_URL_TEST;
    else if (args[0].equals ("IBM_PROD"))
      inquiryUrl = UDDIOperatorURLs.IBM_INQUIRY_URL_PRODUCTION;
    else if (args[0].equals ("MS_TEST"))
      inquiryUrl = UDDIOperatorURLs.MICROSOFT_INQUIRY_URL_TEST;
    else if (args[0].equals ("MS_PROD"))
      inquiryUrl = UDDIOperatorURLs.MICROSOFT_INQUIRY_URL_PRODUCTION;
    else
      inquiryUrl = args[0];

    String inquiryMethod = args[1];
    if (!inquiryMethod.equals ("find_business") && !inquiryMethod.equals ("find_service")
        && !inquiryMethod.equals ("find_tmodel"))
      fatal ("Inquiry method '" + inquiryMethod
      + "' not valid- has to be one of find_business|find_service|find_tmodel");

    /* Construct a UDDIProxy object- this represents a UDDI server 
     * and the actions that can be invoked against it.
     */
    uddiProxy = new UDDIProxy ();
    /* Setting for the proxy server- in case you are behind one */
    /*
     SOAPHTTPConnection soapHTTPConnection = new SOAPHTTPConnection ();
     soapHTTPConnection.setUserName ("proxy-username");
     soapHTTPConnection.setPassword ("proxy-password");
     soapHTTPConnection.setProxyHost ("proxyhostname");
     soapHTTPConnection.setProxyPort (8080);
     uddiProxy.setTransport (soapHTTPConnection);
    */


    /* Set the inquiry URL */
    try {
      uddiProxy.setInquiryURL (inquiryUrl);
      info ("Using Inquiry URL [" + inquiryUrl + "]");
    } catch (MalformedURLException e) {
      fatal ("Malformed publish URL for the UDDI registry [" 
      + inquiryUrl + "]: " +  e.getMessage());
    }

    if (inquiryMethod.equals ("find_business")) {
      if (args[2].equals ("name"))
        System.out.println (browseBusinessByName (args[3])); 
      else if (args[2].equals ("key"))
        System.out.println (browseBusinessByKey (args[3])); 
      else 
        usage ();
    } else if (inquiryMethod.equals ("find_service")) {
        System.out.println (browseServiceByKey (args[3])); 
    } else if (inquiryMethod.equals ("find_tmodel")) {
      if (args[2].equals ("name"))
        System.out.println (browseTModelByName (args[3])); 
      else if (args[2].equals ("key"))
        System.out.println (browseTModelByKey (args[3])); 
      else 
        usage ();
    } else {
       usage ();
    }

  }
}
