<?xml version="1.0"?>
<xsl:stylesheet
  version='1.0'
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:wsdl="http://schemas.xmlsoap.org/wsdl/"
  xmlns:soap="http://schemas.xmlsoap.org/wsdl/soap/"
  xmlns:mime="http://schemas.xmlsoap.org/wsdl/mime/"
  xmlns:http="http://schemas.xmlsoap.org/wsdl/http/"
  xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
  xmlns:xsd="http://www.w3.org/2000/10/XMLSchema"
  xmlns:x="http://xmlns.rdfinference.org/tools"
  xmlns:rxsd="http://rdfinference.org/xsd.rdfs#"
  xmlns:rwsdl="http://rdfinference.org/wsdl.rdfs#"
>

  <xsl:strip-space elements='*'/>
  <xsl:output method='xml' indent='yes'/>

  <!-- do nothing on unknown elements in general -->
  <xsl:template match='*' priority='-100'/>
  <xsl:template match='text()' priority='-100'/>

  <!-- Very limited XML schema processing -->

  <xsl:template match='xsd:schema'>
    <rdf:Description ID='xsd:{@targetNamespace}'>
      <rxsd:targetNamespace><xsl:value-of select='@targetNamespace'/></rxsd:targetNamespace>
      <xsl:apply-templates/>
    </rdf:Description>
  </xsl:template>

  <xsl:template match='xsd:element'>
    <xsl:param name='parent' select=''/>
    <rxsd:element>
      <rdf:Description ID='xsd.{@name}'>
        <xsl:if test='@type'>
          <rxsd:type><xsl:value-of select='@type'/></rxsd:type>
        </xsl:if>
        <xsl:if test='$parent'>
          <rxsd:parent><xsl:value-of select='$parent'/></rxsd:parent>
        </xsl:if>
      </rdf:Description>
    </rxsd:element>
        <xsl:apply-templates>
          <xsl:with-param name='parent' select='@name'/>
        </xsl:apply-templates>
  </xsl:template>

  <xsl:template match='xsd:all'>
    <xsl:param name='parent' select=''/>
    <xsl:apply-templates>
      <xsl:with-param name='parent' select='$parent'/>
    </xsl:apply-templates>
  </xsl:template>

  <xsl:template match='xsd:complexType'>
    <xsl:param name='parent' select=''/>
    <xsl:apply-templates>
      <xsl:with-param name='parent' select='$parent'/>
    </xsl:apply-templates>
  </xsl:template>


  <!-- On to the WSDL -->

  <xsl:variable name='rdf-ns' select='"http://www.w3.org/1999/02/22-rdf-syntax-ns#"'/>
  <xsl:variable name='wsdl-rdfs' select='"http://rdfinference.org/wsdl.rdfs#"'/>
  <xsl:variable name='def-name' select='/wsdl:definitions/@name'/>

  <xsl:template match='wsdl:definitions'>
    <rdf:RDF>
      <xsl:copy>
        <xsl:attribute name='rdf:ID' namespace='{$rdf-ns}'>
          <xsl:value-of select='@name'/>
        </xsl:attribute>
        <rwsdl:targetNamespace>
          <xsl:value-of select='@targetNamespace'/>
        </rwsdl:targetNamespace>
      </xsl:copy>
      <xsl:apply-templates/>
    </rdf:RDF>
  </xsl:template>

  <xsl:template match='wsdl:types'>
    <!--
    <rwsdl:defined-by rdf:resource="#{$def-name}"/>
    -->
    <xsl:apply-templates/>
  </xsl:template>

  <xsl:template match='wsdl:message|wsdl:portType|wsdl:service'>
    <xsl:copy>
      <xsl:attribute name='rdf:ID' namespace='{$rdf-ns}'>
        <xsl:value-of select='@name'/>
      </xsl:attribute>
      <rwsdl:defined-by rdf:resource="#{$def-name}"/>
      <xsl:apply-templates select='*'/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match='wsdl:part'>
    <xsl:copy>
      <xsl:attribute name='rdf:ID' namespace='{$rdf-ns}'>
        <xsl:value-of select='@name'/>
      </xsl:attribute>
      <xsl:attribute name='rdf:type' namespace='{$rdf-ns}'>
        <xsl:value-of select='concat($wsdl-rdfs, "part")'/>
      </xsl:attribute>
      <xsl:apply-templates select='@*' mode='copy-over'/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match='wsdl:portType/wsdl:operation'>
    <xsl:copy>
      <xsl:attribute name='rdf:ID' namespace='{$rdf-ns}'>
        <xsl:value-of select='@name'/>
      </xsl:attribute>
      <xsl:attribute name='rdf:type' namespace='{$rdf-ns}'>
        <xsl:value-of select='concat($wsdl-rdfs, "operation")'/>
      </xsl:attribute>
      <xsl:apply-templates/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match='wsdl:input|wsdl:output|wsdl:fault'>
    <xsl:copy>
      <xsl:attribute name='rdf:parseType' namespace='{$rdf-ns}'>
        <xsl:value-of select='"resource"'/>
      </xsl:attribute>
      <rwsdl:message>
        <xsl:value-of select='@message'/>
      </rwsdl:message>
    </xsl:copy>
  </xsl:template>

  <xsl:template match='wsdl:binding'>
    <xsl:copy>
      <xsl:attribute name='rdf:ID' namespace='{$rdf-ns}'>
        <xsl:value-of select='@name'/>
      </xsl:attribute>
      <rwsdl:defined-by rdf:resource="#{$def-name}"/>
      <xsl:apply-templates select='@*'/>
      <xsl:apply-templates select='*' mode='copy-other'/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match='*|@*' mode='copy-over'>
    <xsl:if test='not(name() = "name")'>
      <xsl:copy/>
    </xsl:if>
  </xsl:template>

  <xsl:template match='*' mode='copy-over'>
    <xsl:copy>
      <xsl:apply-templates select='@*|*' mode='copy-over'/>
    </xsl:copy>
  </xsl:template>

</xsl:stylesheet>
